@@ -53,7 +53,6 @@ class WebSocketRouterEndpoint(WebSocketEndpoint):
         websocket.state.routes = {}
 
     async def on_disconnect(self, websocket, close_code):
-        # NOTE: The on_disconnect event does not occur with Starlette TestClient
         for route in websocket.state.routes:
             await websocket.state.routes[route].on_disconnect(close_code)
         websocket.state.routes.clear()
@@ -76,7 +75,10 @@ class WebSocketRouterEndpoint(WebSocketEndpoint):
                     self._routes[msg['route']](msg['route'], websocket)
                 )
 
-                await endpoint.on_receive(**msg)
+                disconnect = await endpoint.on_receive(**msg)
+
+                if disconnect:
+                    del wsroutes[msg['route']]
 
         except Exception as ex: # pragma: no cover
             self._logger.exception(f"Exception in Websocket Receive [{ex}]: [{data}]")
